@@ -1,29 +1,30 @@
 # coding: utf-8
 
 # PYTHON IMPORTS
-import os, re
-from types import MethodType
+import os
+import re
+from time import gmtime
+from time import strftime
+from time import localtime
+from time import time
 
 # DJANGO IMPORTS
 from django.shortcuts import render_to_response, HttpResponse
 from django.template import RequestContext as Context
-from django.http import HttpResponseRedirect, HttpResponseBadRequest, Http404
+from django.http import HttpResponseRedirect, HttpResponseBadRequest
 from django.contrib.admin.views.decorators import staff_member_required
 from django.views.decorators.cache import never_cache
 from django.utils.translation import ugettext as _
 from django import forms
 from django.core.urlresolvers import reverse, get_urlconf, get_resolver
 from django.core.paginator import Paginator, InvalidPage, EmptyPage
-from django.utils.encoding import smart_unicode
+from django.utils.encoding import smart_text
 from django.contrib import messages
-from django.views.decorators.csrf import csrf_exempt, csrf_protect
-from django.core.files.base import ContentFile
+from django.views.decorators.csrf import csrf_exempt
 from django.core.files.storage import DefaultStorage, default_storage, FileSystemStorage
-from django.core.exceptions import ImproperlyConfigured
 
 # FILEBROWSER IMPORTS
 from filebrowser.settings import *
-from filebrowser.functions import get_breadcrumbs, get_filterdate, get_settings_var, handle_file_upload, convert_filename
 from filebrowser.templatetags.fb_tags import query_helper
 from filebrowser.base import FileListing, FileObject
 from filebrowser.decorators import path_exists, file_exists
@@ -55,22 +56,22 @@ _sites_cache = {}
 
 def get_site_dict(app_name='filebrowser'):
     """
-    Return a dict with all *deployed* FileBrowser sites that have 
+    Return a dict with all *deployed* FileBrowser sites that have
     a given app_name.
     """
-    if not _sites_cache.has_key(app_name):
+    if not app_name in _sites_cache:
         return {}
     # Get names of all deployed filebrowser sites with a give app_name
     deployed = get_resolver(get_urlconf()).app_dict[app_name]
     # Get the deployed subset from the cache
-    return dict((k,v) for k, v in _sites_cache[app_name].iteritems() if k in deployed)
+    return dict((k, v) for k, v in _sites_cache[app_name].iteritems() if k in deployed)
 
 
 def register_site(app_name, site_name, site):
     """
     Add a site into the site dict.
     """
-    if not _sites_cache.has_key(app_name):
+    if not app_name in _sites_cache:
         _sites_cache[app_name] = {}
     _sites_cache[app_name][site_name] = site
 
@@ -88,55 +89,138 @@ def get_default_site(app_name='filebrowser'):
     app_list = resolver.app_dict[app_name]
     if not name in app_list:
         name = app_list[0]
-    
+
     return get_site_dict()[name]
 
 
+def get_breadcrumbs(query, path):
+    """
+    Get breadcrumbs.
+    """
+
+    breadcrumbs = []
+    dir_query = ""
+    if path:
+        for item in path.split(os.sep):
+            dir_query = os.path.join(dir_query, item)
+            breadcrumbs.append([item, dir_query])
+    return breadcrumbs
+
+
+def get_filterdate(filter_date, date_time):
+    """
+    Get filterdate.
+    """
+
+    returnvalue = ''
+    date_year = strftime("%Y", gmtime(date_time))
+    date_month = strftime("%m", gmtime(date_time))
+    date_day = strftime("%d", gmtime(date_time))
+    if filter_date == 'today' and int(date_year) == int(localtime()[0]) and int(date_month) == int(localtime()[1]) and int(date_day) == int(localtime()[2]):
+        returnvalue = 'true'
+    elif filter_date == 'thismonth' and date_time >= time()-2592000:
+        returnvalue = 'true'
+    elif filter_date == 'thisyear' and int(date_year) == int(localtime()[0]):
+        returnvalue = 'true'
+    elif filter_date == 'past7days' and date_time >= time()-604800:
+        returnvalue = 'true'
+    elif filter_date == '':
+        returnvalue = 'true'
+    return returnvalue
+
+
+def get_settings_var(directory=DIRECTORY):
+    """
+    Get settings variables used for FileBrowser listing.
+    """
+
+    settings_var = {}
+    # Main
+    # Extensions/Formats (for FileBrowseField)
+    settings_var['EXTENSIONS'] = EXTENSIONS
+    settings_var['SELECT_FORMATS'] = SELECT_FORMATS
+    # Versions
+    settings_var['ADMIN_VERSIONS'] = ADMIN_VERSIONS
+    settings_var['ADMIN_THUMBNAIL'] = ADMIN_THUMBNAIL
+    # FileBrowser Options
+    settings_var['MAX_UPLOAD_SIZE'] = MAX_UPLOAD_SIZE
+    # Normalize Filenames
+    settings_var['NORMALIZE_FILENAME'] = NORMALIZE_FILENAME
+    # Convert Filenames
+    settings_var['CONVERT_FILENAME'] = CONVERT_FILENAME
+    # Traverse directories when searching
+    settings_var['SEARCH_TRAVERSE'] = SEARCH_TRAVERSE
+    return settings_var
+
+
+def handle_file_upload(path, file, site):
+    """
+    Handle File Upload.
+    """
+
+    uploadedfile = None
+    try:
+        file_path = os.path.join(path, file.name)
+        uploadedfile = site.storage.save(file_path, file)
+    except Exception, inst:
+        raise inst
+    return uploadedfile
+
+
+def filebrowser_view(view):
+    "Only let staff browse the files"
+    return staff_member_required(never_cache(view))
+
+
 class FileBrowserSite(object):
+    """
+    A filebrowser.site defines admin views for browsing your servers media files.
+    """
 
     def __init__(self, name=None, app_name='filebrowser', storage=default_storage):
         self.name = name
         self.app_name = app_name
         self.storage = storage
-        
+
         self._actions = {}
         self._global_actions = self._actions.copy()
 
         # Register this site in the global site cache
         register_site(self.app_name, self.name, self)
-        
+
         # Per-site settings:
         self.directory = DIRECTORY
-    
+
     def _directory_get(self):
+        "Set directory"
         return self._directory
-    
+
     def _directory_set(self, val):
+        "Get directory"
         self._directory = val
 
     directory = property(_directory_get, _directory_set)
 
-    def filebrowser_view(self, view):
-        return staff_member_required(never_cache(view))
-
     def get_urls(self):
-        from django.conf.urls.defaults import patterns, url, include    
+        "URLs for a filebrowser.site"
+        try:
+            from django.conf.urls import url, patterns
+        except ImportError:
+            # for Django version less then 1.4
+            from django.conf.urls.defaults import url, patterns
 
-        urlpatterns = patterns('',
-    
-            # filebrowser urls (views)
-            url(r'^browse/$', path_exists(self, self.filebrowser_view(self.browse)), name="fb_browse"),
-            url(r'^createdir/', path_exists(self, self.filebrowser_view(self.createdir)), name="fb_createdir"),
-            url(r'^upload/', path_exists(self, self.filebrowser_view(self.upload)), name="fb_upload"),
-            url(r'^delete_confirm/$', file_exists(self, path_exists(self, self.filebrowser_view(self.delete_confirm))), name="fb_delete_confirm"),
-            url(r'^delete/$', file_exists(self, path_exists(self, self.filebrowser_view(self.delete))), name="fb_delete"),
-            url(r'^detail/$', file_exists(self, path_exists(self, self.filebrowser_view(self.detail))), name="fb_detail"),
-            url(r'^version/$', file_exists(self, path_exists(self, self.filebrowser_view(self.version))), name="fb_version"),
-            # non-views
+        # filebrowser urls (views)
+        urlpatterns = patterns(
+            '',
+            url(r'^browse/$', path_exists(self, filebrowser_view(self.browse)), name="fb_browse"),
+            url(r'^createdir/', path_exists(self, filebrowser_view(self.createdir)), name="fb_createdir"),
+            url(r'^upload/', path_exists(self, filebrowser_view(self.upload)), name="fb_upload"),
+            url(r'^delete_confirm/$', file_exists(self, path_exists(self, filebrowser_view(self.delete_confirm))), name="fb_delete_confirm"),
+            url(r'^delete/$', file_exists(self, path_exists(self, filebrowser_view(self.delete))), name="fb_delete"),
+            url(r'^detail/$', file_exists(self, path_exists(self, filebrowser_view(self.detail))), name="fb_detail"),
+            url(r'^version/$', file_exists(self, path_exists(self, filebrowser_view(self.version))), name="fb_version"),
             url(r'^upload_file/$', staff_member_required(csrf_exempt(self._upload_file)), name="fb_do_upload"),
-            
         )
-
         return urlpatterns
 
     def add_action(self, action, name=None):
@@ -188,21 +272,20 @@ class FileBrowserSite(object):
 
     @property
     def urls(self):
+        "filebrowser.site URLs"
         return self.get_urls(), self.app_name, self.name
 
     def browse(self, request):
-        """
-        Browse Files/Directories.
-        """
-
+        "Browse Files/Directories."
         filter_re = []
         for exp in EXCLUDE:
-           filter_re.append(re.compile(exp))
-        for k,v in VERSIONS.iteritems():
-            exp = (r'_%s(%s)') % (k, '|'.join(EXTENSION_LIST))
+            filter_re.append(re.compile(exp))
+        for k, v in VERSIONS.iteritems():
+            exp = (r'_%s(%s)$') % (k, '|'.join(EXTENSION_LIST))
             filter_re.append(re.compile(exp))
 
         def filter_browse(item):
+            "Defining a browse filter"
             filtered = item.filename.startswith('.')
             for re_prefix in filter_re:
                 if re_prefix.search(item.filename):
@@ -210,30 +293,31 @@ class FileBrowserSite(object):
             if filtered:
                 return False
             return True
-        
+
         query = request.GET.copy()
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
-        
-        filelisting = FileListing(path,
+
+        filelisting = FileListing(
+            path,
             filter_func=filter_browse,
             sorting_by=query.get('o', DEFAULT_SORTING_BY),
             sorting_order=query.get('ot', DEFAULT_SORTING_ORDER),
             site=self)
-        
+
         files = []
         if SEARCH_TRAVERSE and query.get("q"):
             listing = filelisting.files_walk_filtered()
         else:
             listing = filelisting.files_listing_filtered()
-        
+
         # If we do a search, precompile the search pattern now
         do_search = query.get("q")
         if do_search:
             re_q = re.compile(query.get("q").lower(), re.M)
-        
+
         filter_type = query.get('filter_type')
         filter_date = query.get('filter_date')
-        
+
         for fileobject in listing:
             # date/type filter
             append = False
@@ -245,17 +329,17 @@ class FileBrowserSite(object):
             # append
             if append:
                 files.append(fileobject)
-        
+
         filelisting.results_total = len(listing)
         filelisting.results_current = len(files)
-        
+
         p = Paginator(files, LIST_PER_PAGE)
         page_nr = request.GET.get('p', '1')
         try:
             page = p.page(page_nr)
         except (EmptyPage, InvalidPage):
             page = p.page(p.num_pages)
-        
+
         return render_to_response('filebrowser/index.html', {
             'p': p,
             'page': page,
@@ -269,13 +353,11 @@ class FileBrowserSite(object):
         }, context_instance=Context(request, current_app=self.name))
 
     def createdir(self, request):
-        """
-        Create Directory.
-        """
+        "Create Directory"
         from filebrowser.forms import CreateDirForm
         query = request.GET
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
-        
+
         if request.method == 'POST':
             form = CreateDirForm(path, request.POST, filebrowser_site=self)
             if form.is_valid():
@@ -283,8 +365,6 @@ class FileBrowserSite(object):
                 try:
                     signals.filebrowser_pre_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'], site=self)
                     self.storage.makedirs(server_path)
-                    # os.mkdir(server_path)
-                    # os.chmod(server_path, 0775) # ??? PERMISSIONS
                     signals.filebrowser_post_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'], site=self)
                     messages.add_message(request, messages.SUCCESS, _('The Folder %s was successfully created.') % form.cleaned_data['name'])
                     redirect_url = reverse("filebrowser:fb_browse", current_app=self.name) + query_helper(query, "ot=desc,o=date", "ot,o,filter_type,filter_date,q,p")
@@ -296,7 +376,7 @@ class FileBrowserSite(object):
                         form.errors['name'] = forms.util.ErrorList([_('Error creating folder.')])
         else:
             form = CreateDirForm(path, filebrowser_site=self)
-        
+
         return render_to_response('filebrowser/createdir.html', {
             'form': form,
             'query': query,
@@ -306,15 +386,11 @@ class FileBrowserSite(object):
             'breadcrumbs_title': _(u'New Folder'),
             'filebrowser_site': self
         }, context_instance=Context(request, current_app=self.name))
-    
 
     def upload(self, request):
-        """
-        Multipe File Upload.
-        """
+        "Multipe File Upload."
         query = request.GET
-        path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
-        
+
         return render_to_response('filebrowser/upload.html', {
             'query': query,
             'title': _(u'Select files to upload'),
@@ -325,14 +401,13 @@ class FileBrowserSite(object):
         }, context_instance=Context(request, current_app=self.name))
 
     def delete_confirm(self, request):
-        """
-        Delete existing File/Directory.
-        """
+        "Delete existing File/Directory."
         query = request.GET
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
         fileobject = FileObject(os.path.join(path, query.get('filename', '')), site=self)
         if fileobject.filetype == "Folder":
-            filelisting = FileListing(os.path.join(path, fileobject.filename),
+            filelisting = FileListing(
+                os.path.join(path, fileobject.filename),
                 sorting_by=query.get('o', 'filename'),
                 sorting_order=query.get('ot', DEFAULT_SORTING_ORDER),
                 site=self)
@@ -345,7 +420,7 @@ class FileBrowserSite(object):
         else:
             filelisting = None
             additional_files = None
-        
+
         return render_to_response('filebrowser/delete_confirm.html', {
             'fileobject': fileobject,
             'filelisting': filelisting,
@@ -359,13 +434,11 @@ class FileBrowserSite(object):
         }, context_instance=Context(request, current_app=self.name))
 
     def delete(self, request):
-        """
-        Delete existing File/Directory.
-        """
+        "Delete existing File/Directory."
         query = request.GET
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
         fileobject = FileObject(os.path.join(path, query.get('filename', '')), site=self)
-        
+
         if request.GET:
             try:
                 signals.filebrowser_pre_delete.send(sender=request, path=fileobject.path, name=fileobject.filename, site=self)
@@ -382,14 +455,13 @@ class FileBrowserSite(object):
     def detail(self, request):
         """
         Show detail page for a file.
-        
         Rename existing File/Directory (deletes existing Image Versions/Thumbnails).
         """
         from filebrowser.forms import ChangeForm
         query = request.GET
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
         fileobject = FileObject(os.path.join(path, query.get('filename', '')), site=self)
-        
+
         if request.method == 'POST':
             form = ChangeForm(request.POST, path=path, fileobject=fileobject, filebrowser_site=self)
             if form.is_valid():
@@ -422,7 +494,7 @@ class FileBrowserSite(object):
                     form.errors['name'] = forms.util.ErrorList([_('Error.')])
         else:
             form = ChangeForm(initial={"name": fileobject.filename}, path=path, fileobject=fileobject, filebrowser_site=self)
-        
+
         return render_to_response('filebrowser/detail.html', {
             'form': form,
             'fileobject': fileobject,
@@ -437,11 +509,12 @@ class FileBrowserSite(object):
     def version(self, request):
         """
         Version detail.
+        This just exists in order to select a version with a filebrowser–popup.
         """
         query = request.GET
         path = u'%s' % os.path.join(self.directory, query.get('dir', ''))
         fileobject = FileObject(os.path.join(path, query.get('filename', '')), site=self)
-        
+
         return render_to_response('filebrowser/version.html', {
             'fileobject': fileobject,
             'query': query,
@@ -456,17 +529,12 @@ class FileBrowserSite(object):
         if request.method == "POST":
             folder = request.GET.get('folder', '')
 
-            if request.is_ajax(): # Advanced (AJAX) submission
-                filedata = ContentFile(request.raw_post_data)
-            else: # Basic (iframe) submission
-                if len(request.FILES) != 1:
-                    raise Http404('Invalid request! Multiple files included.')
-                filedata = request.FILES.values()[0]
+            if len(request.FILES) == 0:
+                return HttpResponseBadRequest('Invalid request! No files included.')
+            if len(request.FILES) > 1:
+                return HttpResponseBadRequest('Invalid request! Multiple files included.')
 
-            try:
-                filedata.name = convert_filename(request.GET['qqfile'])
-            except KeyError:
-                return HttpResponseBadRequest('Invalid request! No filename given.')
+            filedata = request.FILES.values()[0]
 
             fb_uploadurl_re = re.compile(r'^.*(%s)' % reverse("filebrowser:fb_upload", current_app=self.name))
             folder = fb_uploadurl_re.sub('', folder)
@@ -478,36 +546,33 @@ class FileBrowserSite(object):
             # Check for name collision with a directory
             if file_already_exists and self.storage.isdir(file_name):
                 ret_json = {'success': False, 'filename': filedata.name}
-                return HttpResponse(json.dumps(ret_json)) 
-            
-            signals.filebrowser_pre_upload.send(sender=request, path=request.POST.get('folder'), file=filedata, site=self)
+                return HttpResponse(json.dumps(ret_json))
+
+            signals.filebrowser_pre_upload.send(sender=request, path=folder, file=filedata, site=self)
             uploadedfile = handle_file_upload(path, filedata, site=self)
-            
+
             if file_already_exists and OVERWRITE_EXISTING:
-                old_file = smart_unicode(file_name)
-                new_file = smart_unicode(uploadedfile)
+                old_file = smart_text(file_name)
+                new_file = smart_text(uploadedfile)
                 self.storage.move(new_file, old_file, allow_overwrite=True)
             else:
-                file_name = smart_unicode(uploadedfile)
-            
-            signals.filebrowser_post_upload.send(sender=request, path=request.POST.get('folder'), file=FileObject(smart_unicode(file_name), site=self), site=self)
-            
+                file_name = smart_text(uploadedfile)
+                filedata.name = os.path.relpath(file_name, path)
+
+            signals.filebrowser_post_upload.send(sender=request, path=folder, file=FileObject(smart_text(file_name), site=self), site=self)
+
             # let Ajax Upload know whether we saved it or not
             ret_json = {'success': True, 'filename': filedata.name}
             return HttpResponse(json.dumps(ret_json))
 
 storage = DefaultStorage()
-storage.location = MEDIA_ROOT
-storage.base_url = MEDIA_URL
 # Default FileBrowser site
 site = FileBrowserSite(name='filebrowser', storage=storage)
 
 # Default actions
-from actions import *
+from filebrowser.actions import flip_horizontal, flip_vertical, rotate_90_clockwise, rotate_90_counterclockwise, rotate_180
 site.add_action(flip_horizontal)
 site.add_action(flip_vertical)
 site.add_action(rotate_90_clockwise)
 site.add_action(rotate_90_counterclockwise)
 site.add_action(rotate_180)
-
-
